@@ -5,34 +5,25 @@ import json
 import requests
 from threading import Thread
 import time
+from . import _exceptions
 
 class _CloudMixin:
 
     def __init__(self, *, project_id, username="python", session_id=None, cloud_host=None):
-        """
-        A class for cloud interactions.
-        """
         self._session_id = session_id
         self._username = username
         self.project_id = str(project_id)
+        self._ratelimited_until = 0#deals with the 0.1 second rate limit for cloud variable sets
         self._connect(cloud_host=cloud_host)
+        self.cloud_host = cloud_host
 
     def _send_packet(self, packet):
-        """
-        Don't use this. Inner function for sending websocket packets.
-        """
         self.websocket.send(json.dumps(packet) + "\n")
 
     def disconnect(self):
-        """
-        Disconnects from Scratch's cloud var websocket.
-        """
         self.websocket.close()
 
     def _handshake(self):
-        """
-        Don't use this. Inner function for handshaking with Scratch's cloud server.
-        """
         self._send_packet(
             {"method": "handshake", "user": self._username, "project_id": self.project_id}
         )
@@ -40,21 +31,25 @@ class _CloudMixin:
 class CloudConnection(_CloudMixin):
 
     def _connect(self, *, cloud_host):
-        """
-        Don't use this. Inner function for connecting.
-        """
-        self.websocket = websocket.WebSocket()
-        self.websocket.connect(
-            "wss://clouddata.scratch.mit.edu",
-            cookie="scratchsessionsid=" + self._session_id + ";",
-            origin="https://scratch.mit.edu",
-            enable_multithread=True,
-        )
+        try:
+            self.websocket = websocket.WebSocket()
+            self.websocket.connect(
+                "wss://clouddata.scratch.mit.edu",
+                cookie="scratchsessionsid=" + self._session_id + ";",
+                origin="https://scratch.mit.edu",
+                enable_multithread=True,
+            )
+        except Exception:
+            raise(_exceptions.ConnectionError)
 
     def set_var(self, variable, value):
-        """
-        Sets a cloud variable on the connected Scratch project.
-        """
+        if len(value) > 256:
+            raise(_exceptions.InvalidCloudValue)
+        x = value.replace(".", "")
+        if not value.isnumeric():
+            raise(_exceptions.InvalidCloudValue)
+        while self._ratelimited_until + 0.11 > time.time():
+            pass
         try:
             self._send_packet(
                 {
@@ -66,35 +61,36 @@ class CloudConnection(_CloudMixin):
                 }
             )
         except Exception as e:
-            print(e)
             try:
                 self._handshake()
             except Exception:
-                self._connect(cloud_host=None)
+                self._connect(cloud_host=self.cloud_host)
                 self._handshake()
 
-            time.sleep(0.1)
+            time.sleep(0.11)
             self.set_var(variable, value)
+        self._ratelimited_until = time.time()
 
 class TwCloudConnection(_CloudMixin):
 
     def _connect(self, *, cloud_host):
-        """
-        Don't use this. Inner function for connecting.
-        """
-        if cloud_host is None:
-            cloud_host = "wss://clouddata.turbowarp.org/"
-        self.websocket = websocket.WebSocket()
-        self.websocket.connect(
-            cloud_host,
-        )
-        self._handshake()
-
+        try:
+            if cloud_host is None:
+                cloud_host = "wss://clouddata.turbowarp.org/"
+            self.websocket = websocket.WebSocket()
+            self.websocket.connect(
+                cloud_host,
+            )
+            self._handshake()
+        except Exception:
+            raise(_exceptions.ConnectionError)
 
     def set_var(self, variable, value):
-        """
-        Sets a cloud variable on the connected TurboWarp project.
-        """
+        x = value.replace(".", "")
+        if not value.isnumeric():
+            raise(_exceptions.InvalidCloudValue)
+        while self._ratelimited_until + 0.11 > time.time():
+            pass
         try:
             self._send_packet(
                 {
@@ -116,13 +112,16 @@ class TwCloudConnection(_CloudMixin):
 
             time.sleep(0.1)
             self.set_var(variable, value)
+        self._ratelimited_until = time.time()
+
 
 class CloudEvents:
 
-    def __init__(self, *, project_id):
-        """
-        A class for listening to cloud events (variables being set / being created / being deleted).
-        """
+    class Event:
+        def __init__(self, **entries):
+            self.__dict__.update(entries)
+
+    def __init__(self, project_id):
         self.project_id = int(project_id)
         self.data = get_cloud_logs(project_id=self.project_id, limit = 15)
         self._thread = None
@@ -130,9 +129,6 @@ class CloudEvents:
         self._events = {}
 
     def start(self, *, update_interval = 0.1):
-        """
-        Starts the event listener.
-        """
         if self.running is False:
             self.update_interval = update_interval
             self.running = True
@@ -141,9 +137,6 @@ class CloudEvents:
                 self._events["on_ready"]()
 
     def _update(self):
-        """
-        Don't use this. Inner function for updating data saved in the object.
-        """
         while True:
             if self.running:
                 data = get_cloud_logs(project_id = self.project_id, limit = 15)
@@ -152,16 +145,13 @@ class CloudEvents:
                         if activity in self.data:
                             break
                         if "on_"+activity["verb"][:-4] in self._events:
-                            self._events["on_"+activity["verb"][:-4]](user=activity["user"], var=activity["name"][2:], value=activity["value"], timestamp=activity["timestamp"])
+                            self._events["on_"+activity["verb"][:-4]](self.Event(user=activity["user"], var=activity["name"][2:], name=activity["name"][2:], value=activity["value"], timestamp=activity["timestamp"]))
                 self.data = data
             else:
                 return
             time.sleep(self.update_interval)
 
     def stop(self):
-        """
-        Stops the event listener.
-        """
         if self._thread is not None:
             self.running = False
             self.project_id = None
@@ -169,16 +159,10 @@ class CloudEvents:
             self._thread = None
 
     def event(self, function):
-        """
-        Decorator. Will be called when the event listener receives an event.
-        """
         self._events[function.__name__] = function
 # -----
 
 def get_cloud(project_id):
-    """
-    Fetches the cloud variables of a Scratch project.
-    """
     try:
         response = json.loads(requests.get(f"https://clouddata.scratch.mit.edu/logs?projectid={project_id}&limit=100&offset=0").text)
         response.reverse()
@@ -189,12 +173,9 @@ def get_cloud(project_id):
     except Exception:
         return []
 
-def get_cloud_var(project_id, variable):
-    """
-    Fetches a specific cloud variable's value.
-    """
+def get_var(project_id, variable):
     try:
-        response = json.loads(requests.get(f"https://clouddata.scratch.mit.edu/logs?projectid={project_id}&limit=100&offset=0").text)
+        response = json.loads(requests.get(f"https://clouddata.scratch.mit.edu/logs?projectid={project_id}&limit=100&off7set=0").text)
         response = list(filter(lambda k: k["name"] == "☁ "+variable, response))
         if response == []:
             return None
@@ -204,9 +185,6 @@ def get_cloud_var(project_id, variable):
         return []
 
 def get_cloud_logs(project_id, *, filter_by_var_named =None, limit=25, offset=0):
-    """
-    Fetches the cloud logs of a Scratch project.
-    """
     try:
         response = json.loads(requests.get(f"https://clouddata.scratch.mit.edu/logs?projectid={project_id}&limit={limit}&offset={offset}").text)
         if filter_by_var_named is None: return response
