@@ -7,9 +7,6 @@ import math
 class CloudRequests:
 
     def __init__(self, cloud_connection : _cloud.CloudConnection):
-        """
-        A class for sending information to Scratch via cloud variables. Requires the "CloudRequests" Scratch sprite made by TimMcCool.
-        """
         print("\033[1mIf you use CloudRequests in your Scratch project, please credit TimMcCool!\033[0m")
         self.connection = cloud_connection
         self.project_id = cloud_connection.project_id
@@ -17,22 +14,16 @@ class CloudRequests:
         self.current_var = 1
 
     def request(self, function):
-        """
-        Decorator. Will be called when the client receives a request with the function's name. The function's returned data will be sent back to the Scratch project.
-        """
+
         self.requests.append(function)
 
     def event(self, function):
-        """
-        Will be called on an event, like "on_ready".
-        """
+
         if function.__name__ == "on_ready":
             self.on_ready = function
 
     def _respond(self, request_id, response):
-        """
-        Don't use this. Inner function for responding to requests.
-        """
+
         remaining_response = str(response)
 
         i = 0
@@ -64,12 +55,21 @@ class CloudRequests:
 
 
     def run(self):
-        """
-        Runs the client.
-        """
-        self.on_ready()
         self.last_data = _cloud.get_cloud_logs(self.project_id, limit=100)
         self.last_timestamp = 0
+        data = _cloud.get_cloud_logs(self.project_id, limit=100)
+        if data == []:
+            pass
+        else:
+            self.last_timestamp = data[0]["timestamp"]
+
+        try:
+            self.on_ready()
+        except AttributeError:
+            pass
+
+        if self.requests = []:
+            print("Warning: You haven't added any requests!")
 
         while True:
             data = _cloud.get_cloud_logs(self.project_id, limit=100)
@@ -88,8 +88,8 @@ class CloudRequests:
 
                         commands = list(filter(lambda k: k.__name__ == request, self.requests))
                         if len(commands) == 0:
-                            print("Unknown command issued")
-                            continue
+                            print(f"Warning: Client received an unknown request called '{request}'")
+                            self._respond(request_id, Encoding.encode(f"Error: Unknown request"))
                         else:
                             try:
                                 if len(arguments) == 0:
@@ -99,19 +99,27 @@ class CloudRequests:
                                 elif len(arguments) == 2:
                                     output = commands[0](arguments[0],arguments[1])
                                 else:
-                                    print("Too many arguments")
-                                    output = Encoding.encode("Error: Too many arguments")
+                                    print(f"Error: Request {request} failed to parse. Don't use the '&' in your requests.")
+                                    output = Encoding.encode("Error: Request failed to parse.")
+                            except TypeError as e:
+                                self._respond(request_id, Encoding.encode("Error: Client received too many arguments, not enough arguments or invalid arguments"))
+                                print(f"Error: Client received too many arguments, not enough arguments or invalid arguments.\nOriginal error: {e}")
                             except Exception as e:
-                                self._respond(request_id, Encoding.encode("Error: Check the Python script"))
+                                self._respond(request_id, Encoding.encode(f"Error: Check the Python console"))
                                 raise(e)
-                        print(isinstance(output, list))
 
-                        if not isinstance(output, list):
-                            output = Encoding.encode(output)
+                        if len(str(output)) > 3000:
+                            length_output = len(str(output))
+                            print(f"Error: Output of request {request} too large!\nMax. output content length: 3000 characters.\nOutput length of {request}: {length_output} characters")
+                            output = "Error: Output too large, check Python console for details."
+
                         else:
-                            input = output
-                            output = ""
-                            for i in input:
-                                output += Encoding.encode(i)
-                                output += "89"
+                            if not isinstance(output, list):
+                                output = Encoding.encode(output)
+                            else:
+                                input = output
+                                output = ""
+                                for i in input:
+                                    output += Encoding.encode(i)
+                                    output += "89"
                         self._respond(request_id, output)
